"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const loader_utils_1 = require("loader-utils");
const utils_1 = require("./utils");
const util_1 = require("util");
const clone_1 = __importDefault(require("clone"));
const BLOCK_MANIFEST_EXPORT_NAME = 'blockManifest';
const COMPONENT_MANIFEST_EXPORT_NAME = 'componentManifest';
const REQUIRED_BLOCKS_EXPORT_NAME = 'requiredBlocks';
const BLOCK_THEMES_EXPORT_NAME = 'blockThemesManifest';
const COMPONENT_THEMES_EXPORT_NAME = 'componentsThemesManifest';
async function default_1(source) {
    const options = loader_utils_1.getOptions(this);
    const blockManifest = await getCompoundManifest.call(this, Object.assign(Object.assign({}, options), { type: 'block' }));
    const componentManifest = await getCompoundManifest.call(this, Object.assign(Object.assign({}, options), { type: 'component' }));
    const blocksThemesManifest = await getManifest.call(this, Object.assign(Object.assign({}, options), { type: 'blocksThemes' }));
    const componentThemesManifest = await getManifest.call(this, Object.assign(Object.assign({}, options), { type: 'componentsThemes' }));
    return source
        .concat(getCompoundDependencies(blockManifest, blocksThemesManifest, options))
        .concat(getExport(BLOCK_MANIFEST_EXPORT_NAME, blockManifest))
        .concat(getExport(COMPONENT_MANIFEST_EXPORT_NAME, componentManifest))
        .concat(getExport(BLOCK_THEMES_EXPORT_NAME, blocksThemesManifest))
        .concat(getExport(COMPONENT_THEMES_EXPORT_NAME, componentThemesManifest));
}
exports.default = default_1;
async function getCompoundManifest(options) {
    const manifest = await getManifest.call(this, options);
    const components = await getCompoundComponents(manifest.components, options);
    return Object.assign(Object.assign({}, manifest), { components });
}
async function getManifest(options) {
    const path = getManifestPath(options);
    if (!path) {
        return { components: [] };
    }
    try {
        const readAsync = util_1.promisify(this.fs.readFile.bind(this.fs));
        const res = (await readAsync(path)).toString();
        this.dependency(path);
        return JSON.parse(res);
    }
    catch (e) {
        return { components: [] };
    }
}
async function getCompoundComponents(components = [], options) {
    const compounds = components
        .map(component => getCompoundComponent(component, options));
    return Promise.all(compounds);
}
async function getCompoundComponent(component, options) {
    if (!isObject(component.icon)) {
        return Object.assign(Object.assign({}, component), { icon: await utils_1.getIcon(component, options) });
    }
    return component;
}
function getCompoundDependencies(manifest, blocksThemesManifest, options) {
    var _a, _b;
    const currentTheme = options.currentTheme || '';
    const blocksWithTheme = (_b = (_a = blocksThemesManifest === null || blocksThemesManifest === void 0 ? void 0 : blocksThemesManifest.themes) === null || _a === void 0 ? void 0 : _a[currentTheme]) === null || _b === void 0 ? void 0 : _b.blocks;
    let blocks = clone_1.default(manifest.components);
    if (currentTheme && blocksWithTheme) {
        blocks = addThemePathToBlock(blocks, blocksWithTheme);
    }
    const exportDependencies = blocks
        .map(component => getBlockDependency(component, options))
        .join('');
    return `exports.${REQUIRED_BLOCKS_EXPORT_NAME} = {};${exportDependencies}`;
}
function addThemePathToBlock(blocks, blocksWithTheme) {
    const dictionaryThemes = parsingThemesToDictionary(blocksWithTheme);
    return blocks.map(block => {
        var _a;
        if ((_a = dictionaryThemes[block.id]) === null || _a === void 0 ? void 0 : _a.path)
            block.path = dictionaryThemes[block.id].path;
        return block;
    });
}
function parsingThemesToDictionary(arr) {
    const dictionary = {};
    if (!arr)
        return dictionary;
    arr.forEach((element) => {
        if (typeof element === 'object') {
            dictionary[element.id] = element;
            return;
        }
        dictionary[element] = { id: element };
    });
    return dictionary;
}
function getBlockDependency(component, options) {
    const templateLoaderQuery = getTemplateLoaderQuery(options);
    const path = getBlockDependencyPath(component, options);
    return `exports.${REQUIRED_BLOCKS_EXPORT_NAME}['${component.id}'] = require('${templateLoaderQuery}${normalizePath(path)}');`;
}
function getBlockDependencyPath(component, options) {
    const componentPath = component.path || path_1.join(options.blocksPath, component.id);
    return path_1.resolve(options.presentationPath, componentPath, 'index.vue');
}
function getTemplateLoaderQuery(options) {
    return `template-loader?baseDir=${normalizePath(options.presentationPath)}&filePathProps=${encodeURIComponent(JSON.stringify(options.componentsProps.filePathProps))}&componentType=block!`;
}
function getManifestPath(options) {
    switch (options.type) {
        case 'block':
            return options.blockManifestPath;
        case 'component':
            return options.componentManifestPath;
        case 'blocksThemes':
            return options.blocksThemesPath;
        case 'componentsThemes':
            return options.componentsThemesPath;
        default:
            return '';
    }
}
function getExport(name, manifest) {
    return `exports.${name}=${JSON.stringify(manifest)};`;
}
function isObject(type) {
    return typeof type === 'object';
}
function normalizePath(path) {
    const isWindows = path_1.sep === '\\';
    if (isWindows) {
        return path.replace(/\\/g, '\\\\');
    }
    return path;
}
//# sourceMappingURL=manifest-loader.js.map